
// Write the dat file that contains the GifTag and data to draw
// QW 0 - GIFTAG:
//        NLOOP=2,
//        EOP=0, (Follow straight on to Stars GIFtag)
//        PRE=1,
//        PRIM=tri,gouraud,notex,nofog,noalpha,noaa,N/A,ctxt1,unfixed
//        FLG=PACKED
//        NREG=2
//        REGS=0x51
// Loop for NUM_TRIS through
// QW 1 - COLOR 1 (yellow)
// QW 2 - VERTEX 1 (Read from list)
// QW 3 - COLOR 2 (magenta)
// QW 4 - VERTEX 2 (Read from list)
// QW 5 - COLOR 3 (cyan)
// QW 6 - VERTEX 3 (Read from list)
// .
// .
// .
// QW 85 - GIFTAG:
//        NLOOP=NUM_STARS,
//        EOP=1, (Finish after starfield
//        PRE=1,
//        PRIM=point,flat,notex,nofog,noalpha,noaa,N/A,ctxt1,unfixed
//        FLG=PACKED
//        NREG=2
//        REGS=0x51
// Loop for NUM_STARS through
// QW 86 - COLOR 1 (random)
// QW 87 - VERTEX 1 (random)

#include <stdio.h>
#include "packet.h"

#define NUM_TRIS	14
#define NUM_STARS	200

const int vert[NUM_TRIS * 6] = {-150,  -90, -150,  -50, -240, -100,
				-150,  -90, -150,  -50, -220,  -20,
				-240, -100, -200, -100, -240,  100,
				-140,  100, -100,  100, -140, -100,
				-140,  100, -140,   60,  -50,  100,
				 -50, -100,  -50,  -60, -140, -100,
				 -90, -100,  -50, -100,  -50,  100,
				 -40,  100,    0,  100,  -40, -100,
				 -40,  100,  -40,  -40,   50,  100,
				  60,  100,  100,  100,   60, -100,
				  60,  100,   60,   60,  150,  100,
				 190,  -20,  220,    0,  230, -100,
				 180,  100,  220,  100,  200,  -40,
				 180,    0,  210,  -20,  160, -100};

typedef union
{
	u_long128     ul128;
	u_long64      ul64[2];
	u_int         ui32[4];
	float         f32[4];
} qword_t;

int main(int argc, char **argv)
{
	FILE *output;
	int bytes=0;
	int prim=0;
	qword_t texture;
	qword_t color;
	qword_t vertex;
	qword_t reg;
	int i,j;

	u_long128 packet_data[1024];

	gifpacket_t gifpacket;

	for (i=0; i<1024; i++)
	{
		packet_data[i] = 0;
	}

	if (argc!=2)
	{
		fprintf(stderr, "Usage: gen_data output_file\n");
		return -1;
	}

	gifpacket_init(&gifpacket, packet_data);

	prim=GS_SET_PRIM(GS_PRIM_TRI, GOURAUD(1), TEXTURED(0), FOG(0), ABE(0), ALPHA(0), AA(0), CTXT(0), FIX(0));
	gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(NUM_TRIS*3), EOP(0), PRE(1), prim, GIF_PACKED, NREG(2)));
	gifpacket_addgsdata(&gifpacket, 0x51);

	for(i=0, j=0; i<NUM_TRIS; i++)
	{
		color.ui32[0]=0x0;
		color.ui32[1]=0xFF;
		color.ui32[2]=0xFF;
		color.ui32[3]=0x80;
		gifpacket_addgspacked(&gifpacket, color.ul128);

		vertex.ui32[0]=(2048+vert[j++])<<4;
		vertex.ui32[1]=(2048+vert[j++])<<4;
		vertex.ui32[2]=100;
		vertex.ui32[3]=1;
		gifpacket_addgspacked(&gifpacket, vertex.ul128);

		color.ui32[0]=0xFF;
		color.ui32[1]=0x0;
		color.ui32[2]=0xFF;
		color.ui32[3]=0x80;
		gifpacket_addgspacked(&gifpacket, color.ul128);

		vertex.ui32[0]=(2048+vert[j++])<<4;
		vertex.ui32[1]=(2048+vert[j++])<<4;
		vertex.ui32[2]=100;
		vertex.ui32[3]=1;
		gifpacket_addgspacked(&gifpacket, vertex.ul128);

		color.ui32[0]=0xFF;
		color.ui32[1]=0xFF;
		color.ui32[2]=0x0;
		color.ui32[3]=0x80;
		gifpacket_addgspacked(&gifpacket, color.ul128);

		vertex.ui32[0]=(2048+vert[j++])<<4;
		vertex.ui32[1]=(2048+vert[j++])<<4;
		vertex.ui32[2]=100;
		vertex.ui32[3]=1;
		gifpacket_addgspacked(&gifpacket, vertex.ul128);
	}

	prim=GS_SET_PRIM(GS_PRIM_POINT, GOURAUD(0), TEXTURED(0), FOG(0), ABE(0), ALPHA(0), AA(0), CTXT(0), FIX(0));
	gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(NUM_STARS), EOP(1), PRE(1), prim, GIF_PACKED, NREG(2)));
	gifpacket_addgsdata(&gifpacket, 0x51);

	for (i=0; i<NUM_STARS; i++)
	{
		color.ui32[0]=i+50;
		color.ui32[1]=2*i+50;
		color.ui32[2]=3*i+50;
		color.ui32[3]=0x80;
		gifpacket_addgspacked(&gifpacket, color.ul128);

		vertex.ui32[0]=((rand()%640) + 1728) << 4;
		vertex.ui32[1]=((rand()%224) + 1936) << 4;
		vertex.ui32[2]=((rand()%5) + 1) << 4;
		vertex.ui32[3]=1;
		gifpacket_addgspacked(&gifpacket, vertex.ul128);
	}

	gifpacket_terminate(&gifpacket);

	output=fopen(argv[1], "wb");

	if (output==NULL)
	{
		fprintf(stderr, "Could not open file \"%s\"\n", argv[1]);
		return -1;
	}

	fwrite(packet_data, sizeof(packet_data)/sizeof(qword_t), sizeof(qword_t), output);

	bytes=ftell(output);
	fclose(output);

	printf("Written %d bytes to \"%s\"\n", bytes, argv[1]);

	return 0;
}
